#!/usr/bin/env python3

"""The publish script."""

import argparse
import os
import os.path
import re
import subprocess  # nosec
import sys
from pathlib import Path
from re import Match
from typing import Optional, cast

import security_md
import yaml

import tag_publish
import tag_publish.configuration
import tag_publish.lib.docker
import tag_publish.lib.oidc
import tag_publish.publish


def match(tpe: str, base_re: str) -> Optional[Match[str]]:
    """
    Return the match for `GITHUB_REF` basically like: `refs/<tpe>/<base_re>`.

    Arguments:
        tpe: The type of ref we want to match (heads, tag, ...)
        base_re: The regular expression to match the value

    """
    if base_re[0] == "^":
        base_re = base_re[1:]
    if base_re[-1] != "$":
        base_re += "$"
    return re.match(f"^refs/{tpe}/{base_re}", os.environ["GITHUB_REF"])


def main() -> None:
    """Run the publish."""
    parser = argparse.ArgumentParser(description="Publish the project.")
    parser.add_argument("--group", default="default", help="The publishing group")
    parser.add_argument("--version", help="The version to publish to")
    parser.add_argument(
        "--docker-versions",
        help="The versions to publish on Docker registry, comma separated, ex: 'x,x.y,x.y.z,latest'.",
    )
    parser.add_argument("--dry-run", action="store_true", help="Don't do the publish")
    parser.add_argument("--dry-run-tag", help="Don't do the publish, on a tag")
    parser.add_argument("--dry-run-branch", help="Don't do the publish, on a branch")
    parser.add_argument("--dry-run-pull", help="Don't do the publish, on a pull request")
    parser.add_argument(
        "--type",
        help="The type of version, if no argument provided auto-determinate, can be: "
        "rebuild (in case of rebuild), tag, default_branch, stabilization, feature_branch, pull_request "
        "(for pull request)",
    )
    args = parser.parse_args()

    if args.dry_run_tag is not None:
        args.dry_run = True
        os.environ["GITHUB_REF"] = f"refs/tags/{args.dry_run_tag}"
    if args.dry_run_branch is not None:
        args.dry_run = True
        os.environ["GITHUB_REF"] = f"refs/heads/{args.dry_run_branch}"
    if args.dry_run_pull is not None:
        args.dry_run = True
        os.environ["GITHUB_REF"] = f"refs/pull/{args.dry_run_pull}"

    config = tag_publish.get_config()

    # Describe the kind of release we do: rebuild (specified with --type), tag, default_branch,
    # stabilization_branch, feature_branch, pull_request (merge, number)
    version: str = ""
    ref = os.environ.get("GITHUB_REF", "refs/heads/fake-local")
    local = "GITHUB_REF" not in os.environ

    if args.type is not None and args.version is None:
        print("::error::you specified the argument --type but not the --version")
        sys.exit(1)

    version_type = args.type
    github = tag_publish.GH()
    security = tag_publish.get_security_md(github, local)
    transformers = config.get(
        "transformers",
        cast(tag_publish.configuration.Transformers, tag_publish.configuration.TRANSFORMERS_DEFAULT),
    )

    if args.version is not None:
        version = args.version
    elif ref.startswith("refs/tags/"):
        version_type = "tag"
        tag_match = tag_publish.match(
            ref.split("/", 2)[2],
            tag_publish.compile_re(
                transformers.get("tag_to_version", cast(tag_publish.configuration.Transform, [{}])),
            ),
        )
        version = tag_publish.get_value(*tag_match)
    elif ref.startswith("refs/heads/"):
        branch = ref.split("/", 2)[2]
        if branch == github.repo.default_branch:
            version_type = "default_branch"
        elif branch in security.branches():
            version_type = "stabilization_branch"
        else:
            version_type = "feature_branch"

        if version_type in ("default_branch", "stabilization_branch"):
            branch_match = tag_publish.match(
                ref.split("/", 2)[2],
                tag_publish.compile_re(
                    transformers.get("branch_to_version", cast(tag_publish.configuration.Transform, [{}])),
                ),
            )
            version = tag_publish.get_value(*branch_match)
        else:
            version = branch.replace("/", "_")
    elif ref.startswith("refs/pull/"):
        version_type = "pull_request"
        pull_match = tag_publish.match(
            ref.split("/")[2],
            tag_publish.compile_re(
                transformers.get("pull_request_to_version", cast(tag_publish.configuration.Transform, [{}])),
            ),
        )
        version = tag_publish.get_value(*pull_match)
    else:
        print(
            f"WARNING: {ref} is not supported, only ref starting with 'refs/heads/', 'refs/tags/' "
            "or 'refs/pull/' are supported, ignoring",
        )
        sys.exit(0)

    print(f"Create release type {version_type}: {version}" + (" (dry run)" if args.dry_run else ""))

    success = True
    published_payload: list[tag_publish.PublishedPayload] = []

    success &= _handle_pypi_publish(
        args.group,
        args.dry_run,
        config,
        version,
        version_type,
        published_payload,
    )
    success &= _handle_node_publish(
        args.group,
        args.dry_run,
        config,
        version,
        version_type,
        published_payload,
    )
    success &= _handle_docker_publish(
        args.group,
        args.dry_run,
        args.docker_versions,
        config,
        version,
        version_type,
        published_payload,
        security,
    )
    success &= _handle_helm_publish(
        args.group,
        args.dry_run,
        config,
        version,
        version_type,
        github,
        published_payload,
    )
    _trigger_dispatch_events(config, version, version_type, published_payload, github)

    if not success:
        sys.exit(1)


def _handle_pypi_publish(
    group: str,
    dry_run: bool,
    config: tag_publish.configuration.Configuration,
    version: str,
    version_type: str,
    published_payload: list[tag_publish.PublishedPayload],
) -> bool:
    success = True
    pypi_config = config.get("pypi", {})
    if pypi_config:
        if "packages" in pypi_config:
            tag_publish.lib.oidc.pypi_login()

        for package in pypi_config.get("packages", []):
            if package.get("group", tag_publish.configuration.PIP_PACKAGE_GROUP_DEFAULT) == group:
                publish = version_type in pypi_config.get(
                    "versions_type",
                    tag_publish.configuration.PYPI_VERSIONS_DEFAULT,
                )
                folder = package.get("folder", tag_publish.configuration.PYPI_PACKAGE_FOLDER_DEFAULT)
                if dry_run:
                    print(f"{'Publishing' if publish else 'Checking'} '{folder}' to pypi, skipping (dry run)")
                else:
                    success &= tag_publish.publish.pip(package, version, version_type, publish)
                    if publish:
                        published_payload.append({"type": "pypi", "folder": folder})
    return success


def _handle_node_publish(
    group: str,
    dry_run: bool,
    config: tag_publish.configuration.Configuration,
    version: str,
    version_type: str,
    published_payload: list[tag_publish.PublishedPayload],
) -> bool:
    success = True
    node_config = config.get("node", {})
    if node_config:
        if version_type in ("default_branch", "stable_branch"):
            last_tag = (
                subprocess.run(
                    ["git", "describe", "--abbrev=0", "--tags"],
                    check=True,
                    stdout=subprocess.PIPE,
                )
                .stdout.strip()
                .decode()
            )
            commits_number = subprocess.run(
                ["git", "rev-list", "--count", f"{last_tag}..HEAD"],
                check=True,
                stdout=subprocess.PIPE,
            )

            version = f"{last_tag}.{commits_number}"

        for package in node_config.get("packages", []):
            if package.get("group", tag_publish.configuration.NODE_PACKAGE_GROUP_DEFAULT) == group:
                publish = version_type in node_config.get(
                    "versions_type",
                    tag_publish.configuration.NODE_VERSIONS_DEFAULT,
                )
                folder = package.get("folder", tag_publish.configuration.NODE_PACKAGE_FOLDER_DEFAULT)
                for repo_name, repo_config in node_config.get(
                    "repository",
                    cast(
                        dict[str, tag_publish.configuration.NodeRepository],
                        tag_publish.configuration.NODE_REPOSITORY_DEFAULT,
                    ),
                ).items():
                    if dry_run:
                        print(
                            f"{'Publishing' if publish else 'Checking'} '{folder}' to {repo_name}, "
                            "skipping (dry run)",
                        )
                    else:
                        success &= tag_publish.publish.node(
                            package,
                            version,
                            version_type,
                            repo_config,
                            publish,
                            node_config.get("args", tag_publish.configuration.NODE_ARGS_DEFAULT),
                        )
                        if publish:
                            published_payload.append({"type": "node", "folder": folder})
    return success


def _handle_docker_publish(
    group: str,
    dry_run: bool,
    docker_versions: str,
    config: tag_publish.configuration.Configuration,
    version: str,
    version_type: str,
    published_payload: list[tag_publish.PublishedPayload],
    security: security_md.Security,
) -> bool:
    success = True
    docker_config = config.get("docker", {})
    if docker_config:
        sys.stdout.flush()
        sys.stderr.flush()
        if docker_config.get("github_oidc_login", tag_publish.configuration.DOCKER_AUTO_LOGIN_DEFAULT):
            subprocess.run(
                [
                    "docker",
                    "login",
                    "ghcr.io",
                    "--username=github",
                    f"--password={os.environ['GITHUB_TOKEN']}",
                ],
                check=True,
            )

        version_index = security.version_index
        alternate_tag_index = security.alternate_tag_index

        row_index = -1
        if version_index >= 0:
            for index, row in enumerate(security.data):
                if row[version_index] == version:
                    row_index = index
                    break

        alt_tags = set()
        if alternate_tag_index >= 0 and row_index >= 0:
            alt_tags = {
                t.strip() for t in security.data[row_index][alternate_tag_index].split(",") if t.strip()
            }
        if version_index >= 0 and security.data[-1][version_index] == version:
            add_latest = True
            for data in security.data:
                row_tags = {t.strip() for t in data[alternate_tag_index].split(",") if t.strip()}
                if "latest" in row_tags:
                    print("latest found in ", row_tags)
                    add_latest = False
                    break
            if add_latest:
                alt_tags.add("latest")

        images_src: set[str] = set()
        images_full: list[str] = []
        versions = docker_versions.split(",") if docker_versions else [version]
        for image_conf in docker_config.get("images", []):
            if image_conf.get("group", tag_publish.configuration.DOCKER_IMAGE_GROUP_DEFAULT) == group:
                for tag_config in image_conf.get("tags", tag_publish.configuration.DOCKER_IMAGE_TAGS_DEFAULT):
                    tag_src = tag_config.format(version="latest")
                    image_source = f"{image_conf['name']}:{tag_src}"
                    images_src.add(image_source)

                    for name, conf in docker_config.get(
                        "repository",
                        cast(
                            dict[str, tag_publish.configuration.DockerRepository],
                            tag_publish.configuration.DOCKER_REPOSITORY_DEFAULT,
                        ),
                    ).items():
                        for docker_version in versions:
                            if version_type in conf.get(
                                "versions_type",
                                tag_publish.configuration.DOCKER_REPOSITORY_VERSIONS_DEFAULT,
                            ):
                                tags = [
                                    tag_config.format(version=alt_tag)
                                    for alt_tag in [docker_version, *alt_tags]
                                ]

                                if dry_run:
                                    for tag in tags:
                                        print(
                                            f"Publishing {image_conf['name']}:{tag} to {name}, skipping "
                                            "(dry run)",
                                        )
                                else:
                                    success &= tag_publish.publish.docker(
                                        conf,
                                        name,
                                        image_conf,
                                        tag_src,
                                        tags,
                                        images_full,
                                        published_payload,
                                    )

        if dry_run:
            sys.exit(0)

        dpkg_versions_path = Path(".github/dpkg-versions.yaml")
        versions_config, dpkg_config_found = tag_publish.lib.docker.get_versions_config()
        dpkg_success = True
        for image in images_src:
            dpkg_success &= tag_publish.lib.docker.check_versions(versions_config.get(image, {}), image)

        if not dpkg_success:
            current_versions_in_images: dict[str, dict[str, str]] = {}
            if dpkg_config_found:
                with dpkg_versions_path.open(encoding="utf-8") as dpkg_versions_file:
                    current_versions_in_images = yaml.load(dpkg_versions_file, Loader=yaml.SafeLoader)
            for image in images_src:
                if image in current_versions_in_images:
                    current_versions_in_images[image] = dict(current_versions_in_images[image])
                _, versions_image = tag_publish.lib.docker.get_dpkg_packages_versions(image)
                for dpkg_package, package_version in versions_image.items():
                    if dpkg_package not in current_versions_in_images.get(image, {}):
                        current_versions_in_images.setdefault(image, {})[dpkg_package] = str(package_version)
                for dpkg_package in list(current_versions_in_images[image].keys()):
                    if dpkg_package not in versions_image:
                        del current_versions_in_images[image][dpkg_package]
            if dpkg_config_found:
                print(
                    "::error::Some packages are have a greater version in the config raster then "
                    "in the image.",
                )
            print("Current versions of the Debian packages in Docker images:")
            print(yaml.dump(current_versions_in_images, Dumper=yaml.SafeDumper, default_flow_style=False))
            if dpkg_config_found:
                with dpkg_versions_path.open("w", encoding="utf-8") as dpkg_versions_file:
                    yaml.dump(
                        current_versions_in_images,
                        dpkg_versions_file,
                        Dumper=yaml.SafeDumper,
                        default_flow_style=False,
                    )

            if dpkg_config_found:
                success = False
    return success


def _handle_helm_publish(
    group: str,
    dry_run: bool,
    config: tag_publish.configuration.Configuration,
    version: str,
    version_type: str,
    github: tag_publish.GH,
    published_payload: list[tag_publish.PublishedPayload],
) -> bool:
    success = True
    helm_config = config.get("helm", {})
    if helm_config.get("packages"):
        tag_publish.download_application("helm/chart-releaser")

        owner = github.repo.owner.login
        repo = github.repo.name
        commit_sha = (
            subprocess.run(["git", "rev-parse", "HEAD"], check=True, stdout=subprocess.PIPE)
            .stdout.strip()
            .decode()
        )
        if version_type in ("default_branch", "stabilization_branch"):
            last_tag = (
                subprocess.run(
                    ["git", "describe", "--abbrev=0", "--tags"],
                    check=True,
                    stdout=subprocess.PIPE,
                )
                .stdout.strip()
                .decode()
            )
            commits_number = subprocess.run(
                ["git", "rev-list", "--count", f"{last_tag}..HEAD"],
                check=True,
                stdout=subprocess.PIPE,
            )

            version = f"{last_tag}.{commits_number}"

        for package in helm_config["packages"]:
            if package.get("group", tag_publish.configuration.HELM_PACKAGE_GROUP_DEFAULT) == group:
                versions_type = helm_config.get(
                    "versions_type",
                    tag_publish.configuration.HELM_VERSIONS_DEFAULT,
                )
                publish = version_type in versions_type
                folder = package.get("folder", tag_publish.configuration.HELM_PACKAGE_FOLDER_DEFAULT)
                if publish:
                    if dry_run:
                        print(f"Publishing '{folder}' to helm, skipping (dry run)")
                    else:
                        token = os.environ["GITHUB_TOKEN"]
                        success &= tag_publish.publish.helm(folder, version, owner, repo, commit_sha, token)
                        published_payload.append({"type": "helm", "folder": folder})
                else:
                    print(
                        f"::notice::The folder '{folder}' will be published as helm on version types: "
                        f"{', '.join(versions_type)}",
                    )
    return success


def _trigger_dispatch_events(
    config: tag_publish.configuration.Configuration,
    version: str,
    version_type: str,
    published_payload: list[tag_publish.PublishedPayload],
    github: tag_publish.GH,
) -> None:
    for dispatch_config in config.get("dispatch", []):
        repository = dispatch_config.get("repository")
        event_type = dispatch_config.get("event_type", tag_publish.configuration.DISPATCH_EVENT_TYPE_DEFAULT)

        published = {
            "version": version,
            "version_type": version_type,
            "repository": github.repo.full_name,
            "items": published_payload,
        }

        if repository:
            print(f"::group::Triggering {event_type} on {repository}")
            github_repo = github.github.get_repo(repository)
        else:
            print(f"::group::Triggering {event_type}")
            github_repo = github.repo
        print(yaml.dump(published, Dumper=yaml.SafeDumper, default_flow_style=False))
        print("::endgroup::")
        github_repo.create_repository_dispatch(event_type, {"content": published})


if __name__ == "__main__":
    main()



"""The publishing functions."""

import datetime
import json
import os
import re
import subprocess  # nosec
import sys
from pathlib import Path
from typing import Optional

import ruamel
import tomllib

import tag_publish.configuration


def pip(
    package: tag_publish.configuration.PypiPackage,
    version: str,
    version_type: str,
    publish: bool,
) -> bool:
    """
    Publish to pypi.

    Arguments:
        version: The version that will be published
        version_type: Describe the kind of release we do: rebuild (specified using --type), tag,
                    default_branch, stabilization_branch, feature_branch, pull_request (merge, number)
        publish: If False only check the package
        package: The package configuration

    """
    folder = package.get("folder", tag_publish.configuration.PYPI_PACKAGE_FOLDER_DEFAULT)
    print(f"::group::{'Publishing' if publish else 'Checking'} '{folder}' to pypi")
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        env = {}
        env["VERSION"] = version
        env["VERSION_TYPE"] = version_type

        cwd = Path(folder).resolve()

        dist = cwd / "dist"
        if not dist.exists():
            dist.mkdir()
        if (cwd / "setup.py").exists():
            cmd = ["python3", "./setup.py", "egg_info", "--no-date"]
            cmd += (
                ["--tag-build=dev" + datetime.datetime.now(tz=datetime.timezone.utc).strftime("%Y%m%d%H%M%S")]
                if version_type in ("default_branch", "stabilization_branch", "rebuild")
                else []
            )
            cmd.append("bdist_wheel")
        else:
            if not dist.exists():
                dist.mkdir()
            cmd = ["pip", "wheel", "--no-deps", "--wheel-dir=dist", "."]
            if (cwd / "pyproject.toml").exists():
                use_poetry = False
                if "build_command" not in package:
                    with (cwd / "pyproject.toml").open("rb") as project_file:
                        pyproject = tomllib.load(project_file)
                    re_splitter = re.compile(r"[<>=]+")
                    for requirement in pyproject.get("build-system", {}).get("requires", []):
                        requirement_split = re_splitter.split(requirement)
                        if requirement_split[0] in ("poetry", "poetry-core"):
                            use_poetry = True
                            break
                    subprocess.run(
                        ["pip", "install", *pyproject.get("build-system", {}).get("requires", [])],
                        check=True,
                    )
                if use_poetry:
                    env_bash = " ".join([f"{key}={value}" for key, value in env.items()])
                    print(f"Run in {cwd}: {env_bash} poetry build")
                    sys.stdout.flush()
                    sys.stderr.flush()
                    subprocess.run(["poetry", "build"], cwd=cwd, env={**os.environ, **env}, check=True)
                    cmd = []
        if cmd:
            cmd = package.get("build_command", cmd)
            subprocess.check_call(cmd, cwd=cwd, env=env)
        cmd = [
            "twine",
            *(["upload", "--verbose", "--disable-progress-bar"] if publish else ["check"]),
            *[str(f) for f in cwd.glob("dist/*.whl")],
            *[str(f) for f in cwd.glob("dist/*.tar.gz")],
        ]
        subprocess.check_call(cmd)
        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True


def node(
    package: tag_publish.configuration.NodePackage,
    version: str,
    version_type: str,
    repo_config: tag_publish.configuration.NodeRepository,
    publish: bool,
    args: list[str],
) -> bool:
    """
    Publish node package to npm.

    Arguments:
        version: The version that will be published
        version_type: Describe the kind of release we do: rebuild (specified using --type), tag,
                    default_branch, stabilization_branch, feature_branch, pull_request (merge, number)
        repo_config: The repository configuration
        publish: If False only check the package
        package: The package configuration
        github: The GitHub helper
        args: The additional arguments to pass to npm publish

    """
    folder = package.get("folder", tag_publish.configuration.PYPI_PACKAGE_FOLDER_DEFAULT)
    print(f"::group::{'Publishing' if publish else 'Checking'} '{folder}' to npm")
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        if version_type == "tag":
            path = Path(folder) / "package.json"
            with path.open(encoding="utf-8") as open_file:
                package_json = json.load(open_file)
            package_json["version"] = version
            with path.open("w", encoding="utf-8") as open_file:
                open_file.write(json.dumps(package_json, indent=2) + "\n")

        cwd = Path(folder).resolve()

        is_github = repo_config["host"] == "npm.pkg.github.com"
        old_npmrc = None
        npmrc_filename = Path("~/.npmrc").expanduser()
        if is_github:
            old_npmrc = None
            if npmrc_filename.exists():
                with npmrc_filename.open(encoding="utf-8") as open_file:
                    old_npmrc = open_file.read()
            with npmrc_filename.open("w", encoding="utf-8") as open_file:
                open_file.write(f"//npm.pkg.github.com/:_authToken={os.environ['GITHUB_TOKEN']}\n")
                open_file.write("always-auth=true\n")

        subprocess.run(["npm", "set", "registry", f"https://{repo_config['host']}"], cwd=cwd, check=True)
        subprocess.run(["npm", "publish", *([] if publish else ["--dry-run"]), *args], cwd=cwd, check=True)

        if is_github:
            if old_npmrc is None:
                npmrc_filename.unlink()
            else:
                with npmrc_filename.open("w", encoding="utf-8") as open_file:
                    open_file.write(old_npmrc)
        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True


def docker(
    config: tag_publish.configuration.DockerRepository,
    name: str,
    image_config: tag_publish.configuration.DockerImage,
    tag_src: str,
    dst_tags: list[str],
    images_full: list[str],
    published: Optional[list[tag_publish.PublishedPayload]] = None,
) -> bool:
    """
    Publish to a Docker registry.

    config is like:
        server: # The server fqdn

    image_config is like:
        name: # The image name

    Arguments:
        config: The publishing config
        name: The repository name, just used to print messages
        image_config: The image config
        tag_src: The source tag (usually latest)
        dst_tags: Publish using the provided tags
        images_full: The list of published images (with tag), used to build the dispatch event
        version_type: Describe the kind of release we do: rebuild (specified using --type), tag,
                    default_branch, stabilization_branch, feature_branch, pull_request (merge, number)
        published: The list of published artifacts to be filled

    """
    print(
        f"::group::Publishing {image_config['name']} to the {name} registry "
        f"using the tags {', '.join(dst_tags)}",
    )
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        new_images_full = []
        if "host" in config:
            for tag in dst_tags:
                subprocess.run(
                    [
                        "docker",
                        "tag",
                        f"{image_config['name']}:{tag_src}",
                        f"{config['host']}/{image_config['name']}:{tag}",
                    ],
                    check=True,
                )
                new_images_full.append(f"{config['host']}/{image_config['name']}:{tag}")
                if published is not None:
                    published.append(
                        {
                            "type": "docker",
                            "repository": config["host"],
                            "image": image_config["name"],
                            "tag": tag,
                        },
                    )
        else:
            for tag in dst_tags:
                if tag_src != tag:
                    subprocess.run(
                        [
                            "docker",
                            "tag",
                            f"{image_config['name']}:{tag_src}",
                            f"{image_config['name']}:{tag}",
                        ],
                        check=True,
                    )
                    if published is not None:
                        published.append(
                            {
                                "type": "docker",
                                "repository": "docker.io",
                                "image": image_config["name"],
                                "tag": tag,
                            },
                        )
                new_images_full.append(f"{image_config['name']}:{tag}")

        for image in new_images_full:
            subprocess.run(["docker", "push", image], check=True)
        images_full += new_images_full

        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True


def helm(folder: str, version: str, owner: str, repo: str, commit_sha: str, token: str) -> bool:
    """
    Publish to pypi.

    Arguments:
        folder: The folder to be published
        version: The version that will be published
        owner: The GitHub repository owner
        repo: The GitHub repository name
        commit_sha: The sha of the current commit
        token: The GitHub token

    """
    print(f"::group::Publishing Helm chart from '{folder}' to GitHub release")
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        yaml_ = ruamel.yaml.YAML()
        with (Path(folder) / "Chart.yaml").open(encoding="utf-8") as open_file:
            chart = yaml_.load(open_file)
        chart["version"] = version
        with (Path(folder) / "Chart.yaml").open("w", encoding="utf-8") as open_file:
            yaml_.dump(chart, open_file)
        for index, dependency in enumerate(chart.get("dependencies", [])):
            if dependency["repository"].startswith("https://"):
                subprocess.run(["helm", "repo", "add", str(index), dependency["repository"]], check=True)

        if subprocess.run(["git", "config", "user.email"], capture_output=True, check=False).returncode != 0:  # pylint: disable=subprocess-run-check
            subprocess.run(["git", "config", "--global", "user.email", "tag-publish@gituhb.com"], check=True)
        if subprocess.run(["git", "config", "user.name"], capture_output=True, check=False).returncode != 0:  # pylint: disable=subprocess-run-check
            subprocess.run(
                ["git", "config", "--global", "user.name", "tag-publish (GitHub Action)"],
                check=True,
            )
        subprocess.run(["cr", "package", folder], check=True)
        subprocess.run(
            [
                "cr",
                "upload",
                f"--owner={owner}",
                f"--git-repo={repo}",
                f"--commit={commit_sha}",
                "--release-name-template={{ .Version }}",
                f"--token={token}",
            ],
            check=True,
        )
        if not Path(".cr-index").exists():
            Path(".cr-index").mkdir()
        subprocess.run(
            [
                "cr",
                "index",
                f"--owner={owner}",
                f"--git-repo={repo}",
                f"--charts-repo=https://{owner}.github.io/{repo}",
                "--push",
                "--release-name-template={{ .Version }}",
                f"--token={token}",
            ],
            check=True,
        )
        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True
